<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Models;

use Iqrf\Repository\Entities\Product;
use Iqrf\Repository\Exceptions\CorruptedData;
use Iqrf\Repository\Exceptions\DataNotFound;
use Iqrf\Repository\Exceptions\ProductNotFound;
use Iqrf\Repository\Exceptions\ServiceUnavailable;

/**
 * Product manager
 */
final class ProductManager extends BaseManager {

	/**
	 * Products path
	 */
	private const PATH = 'products';

	/**
	 * Lists all products
	 * @return array<Product> All products
	 * @throws CorruptedData
	 * @throws ServiceUnavailable
	 */
	public function list(): array {
		$response = $this->apiClient->sendApiRequest(self::PATH);
		foreach ($response as &$item) {
			$item = Product::fromArray($item);
		}
		return $response;
	}

	/**
	 * Returns the product by its HWPID
	 * @param int $hwpid Product HWPID
	 * @return Product Product
	 * @throws CorruptedData
	 * @throws ProductNotFound
	 * @throws ServiceUnavailable
	 */
	public function get(int $hwpid): Product {
		try {
			$response = $this->apiClient->sendApiRequest(self::PATH . '/' . $hwpid);
		} catch (DataNotFound $e) {
			throw new ProductNotFound();
		}
		return Product::fromArray($response);
	}

}
